// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:developer';

import 'package:authentication_bloc_example/app/app.dart';
import 'package:authentication_bloc_example/constants.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class _NameInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (name) => context
              .read<SignUpCubit>()
              .dataChanged(formFieldName, Name.dirty(name)),
          keyboardType: TextInputType.name,
          decoration: InputDecoration(
            labelText: 'name',
            helperText: '',
            errorText:
                state.data.input(formFieldName).invalid ? 'invalid name' : null,
          ),
        );
      },
    );
  }
}

class _PhoneInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (phone) => context
              .read<SignUpCubit>()
              .dataChanged(formFieldPhone, Phone.dirty(phone)),
          keyboardType: TextInputType.phone,
          decoration: InputDecoration(
            labelText: 'phone',
            helperText: '',
            errorText: state.data.input(formFieldPhone).invalid
                ? 'invalid phone'
                : null,
          ),
        );
      },
    );
  }
}

class _SirenInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (siren) => context
              .read<SignUpCubit>()
              .dataChanged(formFieldSiren, Siren.dirty(siren)),
          keyboardType: TextInputType.number,
          decoration: InputDecoration(
            labelText: 'siren',
            helperText: '',
            errorText: state.data.input(formFieldSiren).invalid
                ? 'invalid SIREN'
                : null,
          ),
        );
      },
    );
  }
}

class _IbanInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (iban) => context
              .read<SignUpCubit>()
              .dataChanged(formFieldIban, Iban.dirty(iban)),
          keyboardType: TextInputType.text,
          decoration: InputDecoration(
            labelText: 'iban',
            helperText: '',
            errorText:
                state.data.input(formFieldIban).invalid ? 'invalid IBAN' : null,
          ),
        );
      },
    );
  }
}

class _EmailInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      buildWhen: (previous, current) => previous.email != current.email,
      builder: (context, state) {
        return TextField(
          onChanged: (email) => context.read<SignUpCubit>().emailChanged(email),
          keyboardType: TextInputType.emailAddress,
          decoration: InputDecoration(
            labelText: 'email',
            helperText: '',
            errorText: state.email.invalid ? 'invalid email' : null,
          ),
        );
      },
    );
  }
}

class _PasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (password) {
            context.read<SignUpCubit>().passwordChanged(password);
            context.read<SignUpCubit>().dataChanged(
                  formFieldConfirmedPassword,
                  ConfirmedPassword.dirty(
                    password: password,
                    value: context
                        .read<SignUpCubit>()
                        .state
                        .data
                        .input(formFieldConfirmedPassword)
                        .value,
                  ),
                );
          },
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'password',
            helperText: '',
            errorText: state.password.invalid ? 'invalid password' : null,
          ),
        );
      },
    );
  }
}

class _ConfirmPasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return TextField(
          onChanged: (confirmPassword) => context
              .read<SignUpCubit>()
              .dataChanged(
                formFieldConfirmedPassword,
                ConfirmedPassword.dirty(
                  password: context.read<SignUpCubit>().state.password.value,
                  value: confirmPassword,
                ),
              ),
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'confirm password',
            helperText: '',
            errorText: state.data.input(formFieldConfirmedPassword).invalid
                ? 'passwords do not match'
                : null,
          ),
        );
      },
    );
  }
}

class _CheckIsProInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: const Text('Are you a pro?'),
      trailing: BlocBuilder<SignUpCubit, SignUpState>(
        builder: (context, state) {
          return Checkbox(
              value: state.data.input<bool>(formFieldPro).value,
              onChanged: (isPro) {
                final value =
                    isPro!; // tristate is false, so value can't be null

                context.read<SignUpCubit>().dataChanged(
                      formFieldPro,
                      Boolean.dirty(value: value),
                    );

                if (value) {
                  context.read<SignUpCubit>().updateFormData(
                      App.getProFormData(),
                      operation: SetOperation.union);
                } else {
                  context.read<SignUpCubit>().updateFormData(
                      App.getNormalFormData(),
                      operation: SetOperation.intersection);
                }
              });
        },
      ),
    );
  }
}

class _SignUpButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      buildWhen: (previous, current) => previous.status != current.status,
      builder: (context, state) {
        return state.status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: state.status.isValidated
                    ? () => context.read<SignUpCubit>().signUpFormSubmitted()
                    : null,
                child: const Text('SIGN UP'),
              );
      },
    );
  }
}

class _DebugButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<SignUpCubit, SignUpState>(
      builder: (context, state) {
        return ElevatedButton(
          onPressed: () {
            // log(state.toString());
            log(state.data.toMap().toString());
          },
          child: const Text('DEBUG'),
        );
      },
    );
  }
}

class SignUpForm extends StatelessWidget {
  const SignUpForm({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocListener<SignUpCubit, SignUpState>(
      listener: (context, state) {
        if (state.status.isSubmissionSuccess) {
          Navigator.of(context).pop();
        } else if (state.status.isSubmissionFailure) {
          ScaffoldMessenger.of(context)
            ..hideCurrentSnackBar()
            ..showSnackBar(
              SnackBar(content: Text(state.errorMessage ?? 'Sign Up Failure')),
            );
        }
      },
      child: SizedBox(
        height: MediaQuery.of(context).size.height,
        child: Align(
          alignment: const Alignment(0, -1 / 3),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              _NameInput(),
              const SizedBox(height: 8),
              _PhoneInput(),
              const SizedBox(height: 8),
              _CheckIsProInput(),
              const SizedBox(height: 8),
              BlocBuilder<SignUpCubit, SignUpState>(
                builder: (context, state) {
                  if (state.data.input<bool>(formFieldPro).value) {
                    return Column(children: [
                      _SirenInput(),
                      const SizedBox(height: 8),
                      _IbanInput(),
                      const SizedBox(height: 8),
                    ]);
                  }
                  return const SizedBox.shrink();
                },
              ),
              _EmailInput(),
              const SizedBox(height: 8),
              _PasswordInput(),
              const SizedBox(height: 8),
              _ConfirmPasswordInput(),
              const SizedBox(height: 8),
              _SignUpButton(),
              const SizedBox(height: 8),
              _DebugButton(),
            ],
          ),
        ),
      ),
    );
  }
}
