// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:io';

import 'package:wyatt_cli_toolbox/src/prompt/control.dart';
import 'package:wyatt_cli_toolbox/src/prompt/keys.dart';
import 'package:wyatt_cli_toolbox/src/sequences/sequences.dart';
import 'package:wyatt_cli_toolbox/src/stylish/stylish.dart';

class ListChooser {
  ListChooser(this.options) {
    if (stdin.hasTerminal) {
      // lineMode must be true to set echoMode in Windows
      // see https://github.com/dart-lang/sdk/issues/28599
      stdin.echoMode = false;
      stdin.lineMode = false;
    }
  }

  List<String> options;

  String choose() {
    int input;
    int index = 0;

    _renderList(0, initial: true);

    while ((input = _userInput()) != ENTER) {
      if (input == ARROW_UP) {
        if (index > 0) {
          index--;
        }
      } else if (input == ARROW_DOWN) {
        if (index < options.length - 1) {
          index++;
        }
      }
      _renderList(index);
    }
    _resetStdin();
    return options[index];
  }

  void _deletePreviousList() {
    for (int i = 0; i < options.length; i++) {
      Sequences.write(
        Sequences.escMoveUp(1) + Sequences.escClearLineFromCursorToEnd(),
      );
    }
  }

  void _renderList(int index, {bool initial = false}) {
    if (!initial) {
      _deletePreviousList();
    }
    for (int i = 0; i < options.length; i++) {
      if (i == index) {
        stdout.writeln('${Control.rightIndicator()} ${options[i].blue()}');
        continue;
      }
      stdout.writeln('  ${options[i]}');
    }
  }

  void _resetStdin() {
    if (stdin.hasTerminal) {
      // Order is important here
      stdin.lineMode = true;
      stdin.echoMode = true;
    }
  }

  // TODO(hpcl): improve keycode parsing for multiple bytes codes
  int _userInput() {
    final int navigationResult = stdin.readByteSync();
    if (navigationResult < 0) {
      return navigationResult;
    }

    if (Platform.isWindows) {
      if (navigationResult == Keys.enter) {
        return ENTER;
      }
      if (navigationResult == Keys.space) {
        return SPACE;
      }
      if (navigationResult == Keys.arrowUp[0]) {
        return ARROW_UP;
      }
      if (navigationResult == Keys.arrowDown[0]) {
        return ARROW_DOWN;
      } else {
        return navigationResult;
      }
    } else {
      if (navigationResult == ENTER) {
        return ENTER;
      }
      if (navigationResult == SPACE) {
        return SPACE;
      }

      final int anotherByte = stdin.readByteSync();
      if (anotherByte == ENTER) {
        return ENTER;
      }
      if (anotherByte == SPACE) {
        return SPACE;
      }

      final int input = stdin.readByteSync();
      return input;
    }
  }
}
